/*
 * Copyright @ 2018 - present 8x8, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jitsi.nlj.util

import java.time.Duration
import java.util.concurrent.ExecutorService
import java.util.concurrent.TimeUnit

class ExecutorShutdownTimeoutException : Exception("Timed out trying to shutdown executor service")
/**
 * Shutdown [executorService] normally via [ExecutorService.shutdown].  If, after [timeout] / 2, the
 * service has still not shutdown, try to stop it more forcefully via [ExecutorService.shutdownNow].  If,
 * after [timeout] / 2, the service has still not shutdown then throw [ExecutorShutdownTimeoutException].
 *
 */
@Throws(ExecutorShutdownTimeoutException::class)
fun ExecutorService.safeShutdown(timeout: Duration) {
    shutdown()
    if (!awaitTermination(timeout.toMillis() / 2, TimeUnit.MILLISECONDS)) {
        shutdownNow()
        if (!awaitTermination(timeout.toMillis() / 2, TimeUnit.MILLISECONDS)) {
            throw ExecutorShutdownTimeoutException()
        }
    }
}
